@@ -1,11 +1,11 @@
 /***************************************************************************
- SocNetV: Social Networks Visualizer 
- version: 1.4
+ SocNetV: Social Network Visualizer 
+ version: 1.9
  Written in Qt
  
                          edge.h  -  description
                              -------------------
-    copyright            : (C) 2005-2014 by Dimitris B. Kalamaras
+    copyright            : (C) 2005-2015 by Dimitris B. Kalamaras
     email                : dimitris.kalamaras@gmail.com
  ***************************************************************************/
 
@@ -48,7 +48,10 @@ class Edge : public QObject, public QGraphicsItem {
 	Q_INTERFACES (QGraphicsItem)
 
 public:
-	Edge(GraphicsWidget *, Node*, Node*, float, int, QString, bool, bool, bool);
+    Edge(GraphicsWidget *, Node*, Node*,
+         const Qt::PenStyle &style,
+         const float &, const int &, const QString &,
+         const bool&, const bool&, const bool &);
     ~Edge();
 	enum { Type = UserType + 2 };
 	int type() const { return Type; }
@@ -64,18 +67,28 @@ public:
 
 	int sourceNodeNumber();
 	int targetNodeNumber();
-	void setWeight( float  w) ;
-	float weight();
-	void setColor( QString str) ;
-	QString color() ;
-	void addWeight (EdgeWeight* canvasWeight  ) ;
-	void clearWeightList();
-	float width(); 
-	void showArrows(bool);
-	void toggleAntialiasing(bool);
+    void setWeight( const float  &w) ;
 
-	void makeReciprocal();
-	void unmakeReciprocal();
+    float weight() const;
+    void addWeight (EdgeWeight* canvasWeight  ) ;
+    void clearWeightList();
+
+    void showArrows(bool);
+    void toggleAntialiasing(bool);
+
+    void makeReciprocal();
+    void unmakeReciprocal();
+    bool isReciprocal();
+
+    float width() const;
+
+    QPen pen() const;
+    void setStyle( const Qt::PenStyle  &style);
+    Qt::PenStyle style() const;
+
+    void setColor( const QString &str) ;
+    QString color() const ;
+    QString colorToPajek();
 
 	QPainterPath shape() const;
 
@@ -93,6 +106,7 @@ private:
 	Node *source, *target;
 	QPointF sourcePoint, targetPoint;
 	qreal m_arrowSize, m_startOffset, m_endOffset;
+    Qt::PenStyle m_style;
 	list<EdgeWeight*> weightList;
 	QString m_color;
 	int eFrom, eTo;
