@@ -1,12 +1,14 @@
 /***************************************************************************
- SocNetV: Social Networks Visualizer
- version: 1.4
+ SocNetV: Social Network Visualizer
+ version: 1.9
  Written in Qt
  
                          graph.h  -  description
                              -------------------
-    copyright            : (C) 2005-2014 by Dimitris B. Kalamaras
+    copyright            : (C) 2005-2015 by Dimitris B. Kalamaras
     email                : dimitris.kalamaras@gmail.com
+    website:             : http://dimitris.apeiro.gr
+    project site         : http://socnetv.sourceforge.net
  ***************************************************************************/
 
 /*******************************************************************************
@@ -29,9 +31,11 @@
 
 
 #include <QObject>
+#include <QDateTime> 	// used in exporting centrality files
 #include <QList>
 #include <QHash>
 #include <QTextStream>
+#include <QThread>
 //FYI: stack is a wrapper around <deque> in C++, see: www.cplusplus.com/reference/stl/stack
 #include <stack>
 #include <map>
@@ -43,8 +47,9 @@
 
 using namespace std;
 
+
 class QPointF;
-class QDateTime;
+
 
 /**	This is the main class for a Graph, used in conjuction with Vertex, Parser and Matrix objects.
 
@@ -64,11 +69,50 @@ typedef QPair <int, pair_f_b > rel_w_bool;
 typedef QHash < int, rel_w_bool > H_edges;
 typedef QHash<QString, bool> H_StrToBool;
 
-typedef QHash <QString, float> H_StrToFloat;
+
+
+
+class Distance
+{
+public:
+    int target;
+    int distance;
+
+    Distance(int t, int dist)
+        : target(t), distance(dist)
+    {
+
+    }
+};
+
+struct Distance1 {
+    int target;
+    int distance;
+};
+
+// implement a min-priority queue
+class CompareDistances {
+    public:
+    bool operator()(Distance& t1, Distance& t2)
+    {
+       if (t1.distance == t2.distance)
+            return t1.target > t2.target;
+       return t1.distance > t2.distance;  //minimum priority
+       // Returns true if t1 is closer than t2
+       // else
+    }
+};
+
+
+
+
+
 
 class Graph:  public QObject{
     Q_OBJECT
-
+    QThread file_parserThread;
+    QThread wc_parserThread;
+    QThread wc_spiderThread;
 public slots:
     int currentRelation();
 
@@ -83,11 +127,12 @@ public slots:
 
     void setFileType(int, QString, int,int, bool);
     void removeDummyNode(int);
+    void terminateParserThreads (QString reason);
 
     /** Slots to signals from GraphicsWidget and Parser*/
     void createEdge (int, int, float, QString, int, bool, bool);	//GW and Parser.
     void createEdge (int, int, float, int, bool, bool);		//GW
-    void createEdge (int, int);					//WebCrawler
+    void createEdgeWebCrawler (int, int);					//WebCrawler
     void nodeMovement(bool state, int type, int cW, int cH);		//Called by MW to start movement
 
     void slotSetEdgeVisibility(int relation, int, int, bool);
@@ -95,7 +140,7 @@ public slots:
     //auxiliary createVertex functions
     void createVertex(int i, QPointF p); 				//Called by GW
     void createVertex(int i, int canvasWidth, int canvasHeight); 	//Called by MW
-    void createVertex(QString label, int i) ; 			//Called by WebCrawler
+    void createVertexWebCrawler(QString label, int i) ;
 
     /** Slots to signals from MainWindow */
     void changeRelation(int);
@@ -105,18 +150,21 @@ public slots:
     void filterEdgesByWeight (float, bool);		//Called by MW to filter edges over/under a weight
     void filterEdgesByRelation(int relation, bool status);
 
-    void webCrawl( QString, int, int, bool);	//Called by MW to start a web crawler...
+    void webCrawl(QString, int, int, bool extLinks, bool intLinks);	//Called by MW to start a web crawler...
+    void setGraphChanged(bool changed) { graphModified = changed; }
+
 
 signals:
     /** Signals to MainWindow */
     void updateProgressDialog(int );
     void graphChanged();					//call to update MW widgets
-    void selectedVertex(int);				//notifies MW who is the selected node
 
     void signalFileType (int, QString, int,int, bool);	//notifies MW what we have loaded.
     void statusMessage (QString message);			//updates statusbar message
     void addRelationToMW(QString newRelation);
     void describeDataset(QString);
+    void signalNodeSizesByOutDegree(bool);
+    void signalNodeSizesByInDegree(bool);
 
     /** Signals to GraphicsWidget */
     void drawNode( int ,int,  QString, QString, int, QString, QString, int,
@@ -128,14 +176,21 @@ signals:
     void setEdgeVisibility (int, int, int, bool);			// emitted from each Vertex
     void setVertexVisibility(long int, bool);		//notifies GW to disable a node
     void setNodeSize(long int, int);
+    void setNodeShape(const long int, const QString);
+    void setNodeColor(long int, QString);
+    void setNodeLabel(long int, QString);
     void drawEdgeReciprocal(int, int);				//call GW to draw the edge as symmetric one
+    void changeEdgeColor(long int, long int, QString);
     void addGuideCircle(int, int, int);				//call GW to draw a circular layout line somewhere.
     void addGuideHLine (int);					//call GW to draw a horizontal layout line somewhere.
-    void moveNode(int, int, int);
+    void moveNode(int, qreal, qreal);
 
     /** Signals to Vertice */
     void relationChanged(int);
 
+    /** Signals to Crawler threads */
+    void  operateSpider();
+
 
 public: 	
     /* INIT AND CLEAR*/
@@ -149,7 +204,12 @@ public:
     void setShowNumbersInsideNodes(bool toggle);
 
     /*FILES (READ AND WRITE)*/
-    bool loadGraph ( QString, bool,	int maxWidth, int maxHeight, int format, int two_sm_mode);
+    bool loadGraph (const QString, const QString m_codecName,
+                    const bool,
+                    const int maxWidth,
+                    const int maxHeight,
+                    const int format,
+                    const int two_sm_mode);
 
     bool saveGraph( QString fileName, int fileType,
                     QString networkName, int maxWidth, int maxHeight
@@ -167,15 +227,20 @@ public:
     int hasVertex(QString);
     void removeVertex (long int );
 
-    void setInitVertexSize (long int);
-    void setVertexSize(long int v, int );
+    void setInitVertexSize (const long int);
+    void setVertexSize(const long int &v, const int &newsize );
+    void setAllVerticesSize(const int &newsize);
+    int vertexSize(const long int &v);
 
-    void setInitVertexShape (QString);
-    void setVertexShape(int v, QString shape);
-    QString shape(int v);
+    void setInitVertexShape (const QString);
+    void setVertexShape(const int v, const QString shape);
+    void setAllVerticesShape(const QString shape);
+    QString vertexShape(const int &v);
 
-    void setInitVertexColor (QString color);
-    void setVertexColor(long int v, QString color);
+    void setInitVertexColor (const QString &color);
+    void setVertexColor(const long &v, const QString &color);
+    void setAllVerticesColor(const QString &color);
+    QColor vertexColor(const long int &v);
 
     void setInitVertexNumberColor ( QString color);
     void setInitVertexNumberSize (int size);
@@ -186,11 +251,11 @@ public:
     void setInitVertexLabelColor(QString color);
     void setVertexLabel(int v, QString label);
     void setVertexLabelColor(int v1, QString color);
-    QString label(int);
+    QString vertexLabel(const long int &v1);
 
     void updateVertCoords(int v, int x, int y);
 
-    int vertices() ;
+    int vertices(const bool dropIsolates=false, const bool countAll=false) ;
 
     int outboundEdges (int i) ;
     int inboundEdges (int i) ;
@@ -204,19 +269,39 @@ public:
 
     QList<int> verticesIsolated();
 
+    qreal euclideian_distance(const QPointF &a, const QPointF &b);
+    qreal euclideian_distance(const QPointF &a);
+    int sign(const qreal &D);
+
+    qreal layoutForceDirected_F_rep(const qreal &dist,const qreal &optimalDistance) ;
+    qreal layoutForceDirected_F_att(const qreal &dist,const qreal &optimalDistance) ;
+
+    void layoutForceDirected_Eades_moveNodes(const qreal &c4);
+    void layoutForceDirected_FR_moveNodes(const qreal &temperature) ;
+
+    qreal computeOptimalDistance(const int &Vertices);
+    void compute_angles( const QPointF &Delta,
+                         const qreal &dist,
+                         qreal &angle1,
+                         qreal &angle2,
+                         qreal &degrees1,
+                         qreal &degrees2 );
+
     /* EDGES */
-    float hasEdge (int v1, int v2);
+    int enabledEdges();
+    void edges();
+    float hasArc (const long &v1, const long &v2);
+    bool hasEdge (const int &v1, const long int &v2);
     void removeEdge (int v1, int v2);
 
     bool isWeighted();
 
-    void setEdgeWeight (int v1, int v2, float w);
-    void setInitEdgeColor(QString);
+    void setArcWeight (const long int &v1, const long int &v2, const float &w);
+    void setInitEdgeColor(const QString &);
 
-    void setEdgeColor(long int s, long int t, QString color);
-    QString edgeColor (long int s, long int t);
-
-    int totalEdges ();
+    void setEdgeColor(const long int &v1, const long int &v2, const QString &color);
+    QString edgeColor (const long int &v1, const long int &v2);
+    bool setAllEdgesColor(const QString &color);
 
     float density();
 
@@ -224,8 +309,11 @@ public:
     bool isSymmetric();
     void symmetrize();
 
-    void createAdjacencyMatrix(bool,bool);
-    void invertAdjacencyMatrix();
+    void createAdjacencyMatrix(const bool dropIsolates=false,
+                               const bool considerWeights=true,
+                               const bool inverseWeights=false,
+                               const bool symmetrize=false );
+    bool invertAdjacencyMatrix(const QString &method);
 
 
     /* PRINT OUT TO FILES*/
@@ -234,29 +322,60 @@ public:
     void writeAdjacencyMatrixTo(QTextStream& os);
     void writeAdjacencyMatrix(const QString, const char*);
 
-    void writeInvertAdjacencyMatrix(const QString filename,  const char*);
-    void writeDistanceMatrix(const QString fn, const char*);
-    void writeNumberOfGeodesicsMatrix(const QString fn, const char*);
-    void writeEccentricity(const QString, const bool);
+    void writeInvertAdjacencyMatrix(const QString &filename,
+                                    const QString &,
+                                    const QString &);
+    void writeDistanceMatrix(const QString fn, const char*,
+                             const bool considerWeights,
+                             const bool inverseWeights,
+                             const bool dropIsolates);
+    void writeNumberOfGeodesicsMatrix(const QString fn, const char*,
+                                      const bool considerWeights,
+                                      const bool inverseWeights);
+    void writeEccentricity(const QString, const bool considerWeights,
+                           const bool inverseWeights, const bool dropIsolates);
 
-    friend QTextStream& operator <<  (QTextStream& os, Graph& m);
+ //   friend QTextStream& operator <<  (QTextStream& os, Graph& m);
 
-    void writeCentralityDegree(const QString, const bool);	//Writes the out-degree centralities to a file
-    void writeCentralityCloseness(const QString, const bool);	//Writes the closeness centralities to a file
-    void writeCentralityClosenessInfluenceRange(const QString, const bool);
-    void writeCentralityBetweeness(const QString, const bool);	//Writes the betweeness centralities to a file
-    void writeCentralityGraph(const QString, const bool);		//Writes the Graph centralities to a file
-    void writeCentralityPower(const QString, const bool);		//Writes the Power centralities to a file
-    void writeCentralityStress(const QString, const bool);		//Writes the Stress centralities to a file
-    void writeCentralityEccentricity(const QString, const bool);	//Writes the Eccentr centralities to a file
-    void writeCentralityInformation(const QString);			//Writes the Information centralities to a file
-
-    void writePrestigeDegree(const QString, bool);
-    void writePrestigeProximity(const QString, const bool);
-    void writePrestigePageRank(const QString);
+    void writeCentralityDegree(const QString,
+                               const bool weights,
+                               const bool dropIsolates);
+    void writeCentralityCloseness(const QString,
+                                  const bool weights,
+                                  const bool inverseWeights,
+                                  const bool dropIsolates);
+    void writeCentralityClosenessInfluenceRange(const QString,
+                                                const bool weights,
+                                                const bool inverseWeights,
+                                                const bool dropIsolates);
+    void writeCentralityBetweenness(const QString,
+                                    const bool weights,
+                                    const bool inverseWeights,
+                                    const bool dropIsolates);
+    void writeCentralityPower(const QString,
+                              const bool weigths,
+                              const bool inverseWeights,
+                              const bool dropIsolates);
+    void writeCentralityStress(const QString,
+                               const bool weigths,
+                               const bool inverseWeights,
+                               const bool dropIsolates);
+    void writeCentralityEccentricity(const QString,
+                                     const bool weigths,
+                                     const bool inverseWeights,
+                                     const bool dropIsolates);
+    void writeCentralityInformation(const QString,
+                                    const bool weigths,
+                                    const bool inverseWeights);
+    void writePrestigeDegree(const QString, const bool weights,
+                             const bool dropIsolates);
+    void writePrestigeProximity(const QString, const bool weights,
+                                const bool inverseWeights,
+                                const bool dropIsolates);
+    void writePrestigePageRank(const QString, const bool Isolates=false);
 
 
-    void writeNumberOfCliques(
+    void writeCliqueCensus(
             const QString fileName, const bool considerWeights
             );
 
@@ -266,19 +385,29 @@ public:
 
 
     /* DISTANCES, CENTRALITIES & PROMINENCE MEASURES */
-    int distance( int, int);
-    int diameter();
-    float averageGraphDistance();
+    int distance(const int, const int,
+                 const bool considerWeights, const bool inverseWeights);
+    int diameter(const bool considerWeights, const bool inverseWeights);
+    float averageGraphDistance(const bool considerWeights,
+                               const bool inverseWeights, const bool dropIsolates);
     int connectedness();
 
-    void createDistanceMatrix(bool);
-    void centralityDegree(bool);
-    void centralityInformation();
-    void centralityClosenessInfluenceRange();
+    void createDistanceMatrix(const bool centralities=false,
+                              const bool considerWeights=false,
+                              const bool inverseWeights=true,
+                              const bool dropIsolates=false);
+    void centralityDegree(const bool weights, const bool dropIsolates=false);
+    void centralityInformation(const bool considerWeights=false,
+                               const bool inverseWeights=false);
+    void centralityClosenessInfluenceRange(const bool considerWeights=false,
+                                           const bool inverseWeights=false,
+                                           const bool dropIsolates=false);
 
-    void prestigeDegree(bool);
-    int prestigePageRank();
-    void prestigeProximity();
+    void prestigeDegree(bool, bool);
+    void prestigePageRank(const bool dropIsolates=false);
+    void prestigeProximity(const bool considerWeights=false,
+                           const bool inverseWeights=false,
+                           const bool dropIsolates=false);
 
     /* REACHABILTY AND WALKS */
     int numberOfWalks(int v1, int v2,int length);
@@ -288,14 +417,19 @@ public:
     int reachable(int v1, int v2) ;
     QList<int> influenceRange(int v1);
     QList<int> influenceDomain(int v2);
-    void reachabilityMatrix();
-    void writeReachabilityMatrix(QString fn, QString netName);
+    void reachabilityMatrix(const bool considerWeights=false,
+                            const bool inverseWeights=false,
+                            const bool dropIsolates=false);
+    void writeReachabilityMatrix(QString fn, QString netName,
+                                 const bool dropIsolates=false);
 
 
     float numberOfTriples(int v1);
-    float numberOfCliques(int v1);
-    float numberOfCliques();
-    float clusteringCoefficient(int v1);
+    float countCliquesWith(int source, int size=0);
+
+    bool addClique (const QList<int> &list);
+    float countCliquesOfSize(int size );
+    float localClusteringCoefficient(const long int &v1);
     float clusteringCoefficient ();
 
     bool triadCensus();
@@ -309,21 +443,42 @@ public:
 
     void layoutCircularRandom(double x0, double y0, double maxRadius);
 
-    void layoutCircularByProminenceIndex(
-            double x0, double y0, double maxRadius, int type);
+    void layoutCircularByProminenceIndex(double x0, double y0, double maxRadius,
+                                         int type, const bool considerWeights,
+                                         const bool inverseWeights,
+                                         const bool dropIsolates);
 
-    void layoutLevelByProminenceIndex(
-            double maxWidth, double maxHeight, int type);
+    void layoutLevelByProminenceIndex(double maxWidth, double maxHeight, int type,
+                                      const bool considerWeights,
+                                      const bool inverseWeights,
+                                      const bool dropIsolates);
 
-    void layoutVerticesSizeByProminenceIndex(int index);
+    void layoutVerticesSizeByProminenceIndex(int index,
+                                             const bool considerWeights,
+                                             const bool inverseWeights,
+                                             const bool dropIsolates);
 
-    void layoutForceDirectedSpringEmbedder(bool dynamicMovement);
+    void layoutForceDirectedSpringEmbedder(bool &dynamicMovement);
 
     void layoutForceDirectedFruchtermanReingold(bool dynamicMovement);
 
+
+    /* CRAWLER */
+    void terminateCrawlerThreads (QString reason);
+
     /**RANDOM NETWORKS*/
-    void createRandomNetErdos
-    (int, double);
+
+
+
+    void makeThingsLookRandom();
+
+
+    void createRandomNetErdos (  const int &vert,
+                                 const QString &model,
+                                 const int &edges,
+                                 const float &eprob,
+                                 const QString &mode,
+                                 const bool &diag);
 
     void createRandomNetRingLattice
     (int, int, double, double, double);
@@ -331,6 +486,17 @@ public:
     void createSameDegreeRandomNetwork
     (int, int);
 
+    void createRandomNetScaleFree (const int &n,
+                                    const int &power,
+                                    const int &m0,
+                                    const int &m,
+                                    const float &alpha,
+                                    const QString &mode,
+                                    const double &x0,
+                                    const double &y0,
+                                    const double &radius);
+
+
     void createRandomNetSmallWorld
     (int, int, double, double, double, double);
 
@@ -366,9 +532,10 @@ private:
     */
     Vertices m_graph;
 
-    Parser parser;	//file loader threaded class.
+    Parser *file_parser;	//file loader threaded class.
 
-    WebCrawler crawler;
+    WebCrawler_Parser *wc_parser;
+    WebCrawler_Spider *wc_spider;
 
     /** private member functions */
 
@@ -382,7 +549,10 @@ private:
     void addEdge (int v1, int v2, float w, QString color, int reciprocal);
 
     /** methods used by createDistanceMatrix()  */
-    void BFS(int, bool);	//Breadth-First Search function
+    void BFS(const int s, const bool computeCentralities,
+             const bool dropIsolates);
+    void dijkstra(const int s,const bool computeCentralities,
+                  const bool inverseWeights, const bool dropIsolates);
     void minmax(
                 float C, Vertex *v, float &max, float &min,
                 int &maxNode, int &minNode
@@ -398,8 +568,12 @@ private:
     QList<int>  triadTypeFreqs; 	//stores triad type frequencies
     QList<int>  m_isolatedVerticesList;
     QHash <int, int> influenceRanges, influenceDomains;
-    QHash <int, int> notStronglyConnectedVertices;
-    H_StrToFloat edgesHash; // edges to weight hash. Key format "rel:source>target"
+    QHash <int, int> disconnectedVertices;
+    QHash <int, int> unilaterallyConnectedVertices;
+
+    H_StrToBool cliques_2_Vertex;
+    H_StrToBool cliques_3_Vertex;
+    H_StrToBool cliques_4_Vertex;
 
     Matrix  TM, DM, sumM, invAM, AM, invM;
     Matrix XM, XSM, XRM;
@@ -408,29 +582,37 @@ private:
     /** used in resolveClasses and createDistanceMatrix() */
     H_StrToInt discreteDPs, discreteDCs, discreteCCs, discreteBCs, discreteSCs;
     H_StrToInt discreteIRCCs, discreteECs, discreteEccentricities;
-    H_StrToInt discretePCs, discreteICs,  discretePRCs, discretePPs;
+    H_StrToInt discretePCs, discreteICs,  discretePRPs, discretePPs;
 
     bool calculatedDP, calculatedDC, calculatedCentralities, dynamicMovement;
-    bool calculatedPP, calculatedIRCC;
+    bool calculatedPP, calculatedIRCC, calculatedIC, calculatedPRP;
+    bool calculatedTriad;
 
     int m_precision, m_curRelation;
-    float meanDegree, varianceDegree;
+    float meanDC, varianceDC;
     float meanCC, varianceCC;
     float meanIRCC, varianceIRCC;
+    float meanBC, varianceBC;
+    float meanSC, varianceSC;
+    float meanEC, varianceEC;
+    float meanPC, variancePC;
+    float meanIC, varianceIC;
+    float meanDP, varianceDP;
     float meanPP, variancePP;
+    float meanPRP, variancePRP;
     float minEccentricity, maxEccentricity, sumEccentricity;
-    float minDP, maxDP, sumDP, groupDP;
-    float minDC, maxDC, sumDC, groupDC;
+    float minDP, maxDP, t_sumDP, sumDP, groupDP;
+    float minDC, maxDC, t_sumDC, sumDC, groupDC;
     float minCC, maxCC, nomCC, denomCC, sumCC, groupCC, maxIndexCC;
     float minIRCC, maxIRCC, nomIRCC, denomIRCC, sumIRCC, groupIRCC;
     float minBC, maxBC, nomBC, denomBC, sumBC, groupBC, maxIndexBC;
-    float minPC, maxPC, sumPC, groupPC, maxIndexPC;
+    float minPC, maxPC, nomPC, denomPC, t_sumIC, sumPC, groupPC, maxIndexPC;
     float minSC, maxSC, nomSC, denomSC, sumSC, groupSC, maxIndexSC;
     float minEC, maxEC, nomEC, denomEC, sumEC, groupEC, maxIndexEC;
-    float minIC, maxIC, nomIC, denomIC, sumIC, groupIC, maxIndexIC;
-    float minPRC, maxPRC, nomPRC, denomPRC, sumPRC, groupPRC, maxIndexPRC;
+    float minIC, maxIC, nomIC, denomIC, sumIC, maxIndexIC;
+    float minPRP, maxPRP, nomPRC, denomPRC, t_sumPC, t_sumPRP, sumPRP;
     float minPP, maxPP, nomPP, denomPP, sumPP, groupPP;
-    float minCLC, maxCLC, averageCLC, averageIC, averagePRC, dampingFactor;
+    float minCLC, maxCLC, averageCLC, d_factor;
     int maxNodeCLC, minNodeCLC;
     int classesDP, maxNodeDP, minNodeDP;
     int classesDC, maxNodeDC, minNodeDC;
@@ -442,13 +624,13 @@ private:
     int classesEC, maxNodeEC, minNodeEC;
     int classesEccentricity, maxNodeEccentricity, minNodeEccentricity;
     int classesIC, maxNodeIC, minNodeIC;
-    int classesPRC, maxNodePRC, minNodePRC;
+    int classesPRP, maxNodePRP, minNodePRP;
     int classesPP, maxNodePP, minNodePP;
     int sizeOfComponent;
 
     /** General & initialisation variables */
 
-    long int m_totalEdges, m_totalVertices, graphDiameter, initVertexSize;
+    long int m_totalVertices, graphDiameter, initVertexSize;
     int initVertexLabelSize, initVertexNumberSize;
 
     int isolatedVertices;
