@@ -1,12 +1,12 @@
 /***************************************************************************
- SocNetV: Social Networks Visualizer 
- version: 1.4
+ SocNetV: Social Network Visualizer 
+ version: 1.9
  Written in Qt
 
                         main.cpp  -  description
                              -------------------
     begin                : 9 21:10:04 EET 2005
-    copyright            : (C) 2005-2014 by Dimitris B. Kalamaras
+    copyright            : (C) 2005-2015 by Dimitris B. Kalamaras
     email                : dimitris.kalamaras@gmail.com
  ***************************************************************************/
 
@@ -52,7 +52,7 @@ int main(int argc, char *argv[])
     if ( argc > 1 )     {
         option = argv[1];
 			if (option=="--help" || option=="-h" || option=="--h" ) {
-				cout<<"\nSocial Networks Visualizer v." << qPrintable(VERSION)<< "\n"
+				cout<<"\nSocial Network Visualizer v." << qPrintable(VERSION)<< "\n"
 					<<"\nUsage: socnetv [flags] [file]\n"
 					<<"-h, --help 	Displayes this help message\n"
 					<<"-V, --version	Displays version number\n\n"
@@ -64,12 +64,12 @@ int main(int argc, char *argv[])
 				return -1;
 			}
 			else if (option=="-V" || option=="--version") {
-				cout<<"\nSocial Networks Visualizer v." << qPrintable(VERSION)
+				cout<<"\nSocial Network Visualizer v." << qPrintable(VERSION)
 					<< "\nCopyright Dimitris V. Kalamaras, \nLicense: GPL3\n\n";
 				return -1;
 			}
 			else  {
-				cout<<"\nSocial Networks Visualizer v." << qPrintable(VERSION);
+				cout<<"\nSocial Network Visualizer v." << qPrintable(VERSION);
 				cout<<"\nLoading file: " << qPrintable(option) << "\n\n";
 			}
 		
