@@ -1,12 +1,14 @@
 /***************************************************************************
- SocNetV: Social Networks Visualizer
- version: 1.4
+ SocNetV: Social Network Visualizer
+ version: 1.9
  Written in Qt
  
                          mainwindow.h  -  description
                              -------------------
-    copyright            : (C) 2005-2014 by Dimitris B. Kalamaras
+    copyright            : (C) 2005-2015 by Dimitris B. Kalamaras
     email                : dimitris.kalamaras@gmail.com
+    website:             : http://dimitris.apeiro.gr
+    project site         : http://socnetv.sourceforge.net
  ***************************************************************************/
 
 /*******************************************************************************
@@ -39,10 +41,10 @@
 #include "graph.h"
 #include "filteredgesbyweightdialog.h"
 #include "webcrawlerdialog.h"
+#include "nodeeditdialog.h"
 #include "datasetselectdialog.h"
 
-
-static const QString VERSION="1.4";
+static const QString VERSION="1.9";
 
 /**
   * This Class is the base class. It sets up the main
@@ -67,13 +69,18 @@ class QComboBox;
 class QGroupBox;
 class QTabWidget;
 class QSpinBox;
+class PreviewForm;
+
+class RandErdosRenyiDialog;
+class RandSmallWorldDialog;
+class RandScaleFreeDialog;
 
 
 class MainWindow : public QMainWindow
 {
     Q_OBJECT
 
-public:		/**PUBLIC FUNCTIONS NOT VISIBLE BY OTHER WIDGETS NOR BY SLOT/LINK MECHANISM */
+public:
     GraphicsWidget *graphicsWidget;
 
     MainWindow(const QString &f);
@@ -87,19 +94,20 @@ public:		/**PUBLIC FUNCTIONS NOT VISIBLE BY OTHER WIDGETS NOR BY SLOT/LINK MECHA
     void initNet();
     void initView();
 
+    void setLastPath(QString filePath);
+    QString getLastPath();
     void createFortuneCookies();
     void createTips();
-    void makeThingsLookRandom();
 
     bool showLabels();
     bool showNumbersInsideNodes();
     bool showNumbers();
 
-    // Main network file loader method
-    bool loadNetworkFile ( QString, int );
+    // Main network file loader methods
+    bool previewNetworkFile(QString , int );
+    bool loadNetworkFile ( const QString, const QString, const int );
 
-
-    int activeLinks();
+    int activeEdges();
     int activeNodes();
 
     void openContextMenu(const QPointF & mPos);
@@ -139,16 +147,41 @@ public slots:
     //NETWORK MENU
     void slotOpenTextEditor();
     void slotViewNetworkFile();
+
+    void findCodecs();
+    void userCodec(const QString, const QString, const int );
+
     void slotViewAdjacencyMatrix();
     void slotShowDataSetSelectDialog();
     void slotRecreateDataSet(QString);
-    void slotCreateRandomNetErdos();
-    void slotCreateSameDegreeRandomNetwork();
-    void slotCreateGaussianRandomNetwork();
-    void slotCreateRandomNetRingLattice();
-    void slotCreateSmallWorldRandomNetwork();
+    void slotCreateRandomErdosRenyi();
+    void createRandomNetErdos( const int N,
+                               const QString model,
+                               const int edges,
+                               const float eprob,
+                               const QString mode,
+                               const bool diag) ;
+    void slotCreateRegularRandomNetwork();
+    void slotCreateRandomGaussian();
+    void slotCreateRandomRingLattice();
+    void slotCreateRandomScaleFree();
+
+    void createScaleFreeNetwork (const int &nodes,
+                                 const int &power,
+                                 const int &initialNodes,
+                                 const int &edgesPerStep,
+                                 const float &zeroAppeal,
+                                 const QString &mode);
+
+    void slotCreateRandomSmallWorld();
+
+    void createSmallWorldNetwork (const int &nodes,
+                                   const int &degree,
+                                   const float &beta,
+                                   const QString &mode,
+                                   const bool &diag);
     void slotShowWebCrawlerDialog();
-    void slotWebCrawl(QString, int, int, bool);
+    void slotWebCrawl(QString, int, int, bool, bool);
 
     void prevRelation();
     void nextRelation();
@@ -156,26 +189,22 @@ public slots:
     void addRelation(QString relationName);
 
     //EDIT MENU
+    void slotSelectAll();
+    void slotSelectNone();
     void slotFindNode();
-    void slotAddLink();
+    void slotAddEdge();
     void slotRemoveNode();
-    void slotRemoveLink();
-    void slotChangeNodeLabel();
-    void slotChangeNodeColor();
-    void slotChangeNodeValue();
-    void slotChangeNodeSize();
-    void slotChangeNodeBox();
-    void slotChangeNodeCircle();
-    void slotChangeNodeTriangle();
-    void slotChangeNodeDiamond();
-    void slotChangeNodeEllipse();
-    void slotChangeLinkLabel();
-    void slotChangeLinkColor();
-    void slotChangeLinkWeight();
+    void slotNodeProperties( const QString, const int, const QString,
+                             const QColor, const QString);
+    void slotRemoveEdge();
+    void slotChangeNodeProperties();
+    void slotChangeEdgeLabel();
+    void slotChangeEdgeColor();
+    void slotChangeEdgeWeight();
     void slotFilterNodes();
-    void slotFilterIsolateNodes();
+    void slotFilterIsolateNodes(bool checked);
     void slotShowFilterEdgesDialog();
-    void slotTransformNodes2Links();
+    void slotTransformNodes2Edges();
     void slotSymmetrize();
 
     // LAYOUT MENU
@@ -199,8 +228,9 @@ public slots:
     void  slotLayoutNodeSizesByInDegree(bool);
 
     //STATISTICS MENU
-    void slotViewDistanceMatrix();
-    void slotViewNumberOfGeodesicsMatrix();
+    void askAboutWeights();
+    void slotDistancesMatrix();
+    void slotGeodesicsMatrix();
     void slotGraphDistance();
     void slotAverageGraphDistance();
     void slotDiameter();
@@ -211,7 +241,7 @@ public slots:
     void slotReachabilityMatrix();
     void slotConnectedness();
 
-    void slotNumberOfCliques();
+    void slotCliqueCensus();
     void slotClusteringCoefficient();
     void slotTriadCensus();
     void slotCheckSymmetry();
@@ -220,7 +250,7 @@ public slots:
     void slotCentralityDegree();
     void slotCentralityCloseness();
     void slotCentralityClosenessInfluenceRange();
-    void slotCentralityBetweeness();
+    void slotCentralityBetweenness();
     void slotCentralityInformation();
     void slotCentralityStress();
     void slotCentralityPower();
@@ -238,15 +268,16 @@ public slots:
     void slotChangeAllNodesShape();
     void slotChangeNumbersSize();
     void slotChangeLabelsSize();
-    void slotDrawLinksThickAsWeights();
-    void slotDrawLinksBezier(bool toggle);
-    void slotDisplayLinksWeightNumbers(bool toggle);
-    void slotDisplayLinks(bool toggle);
-    void slotDisplayLinksArrows(bool toggle);
+    void slotDrawEdgesThickAsWeights();
+    void slotDrawEdgesBezier(bool toggle);
+    void slotDisplayEdgesWeightNumbers(bool toggle);
+    void slotConsiderEdgeWeights(bool);
+    void slotDisplayEdges(bool toggle);
+    void slotDisplayEdgesArrows(bool toggle);
 
     void slotBackgroundColor ();
     void slotAllNodesColor();
-    void slotAllLinksColor();
+    void slotAllEdgesColor();
     void slotAllNumbersColor();
     void slotAllLabelsColor();
 
@@ -264,22 +295,20 @@ public slots:
     void slotHelpAbout();
     void slotAboutQt();
     //PUBLICLY AVAILABLE SLOTS. CALLED FROM GRAPHICSVIEW
-    void selectNone();
     void nodeInfoStatusBar(Node*);
-    void linkInfoStatusBar (Edge*);
+    void edgeInfoStatusBar (Edge*);
     void openNodeContextMenu();
-    void openLinkContextMenu() ;
+    void openEdgeContextMenu() ;
     void windowInfoStatusBar(int, int);
     void graphChanged();
 
-
     //Called by graphicswidget to update node coords in activeGraph
     void updateNodeCoords(int no, int x, int y);
 
     //Called when user pushes the New Node button on the MW
     void addNode();
     //Called by graphicswidget when the user middle-clicks
-    void addLink (int v1, int v2, float weight);
+    void addEdge (int v1, int v2, float weight);
     //Called by graphicswidget when the user double-clicks
     void addNodeWithMouse(int, QPointF);
 
@@ -300,6 +329,8 @@ public slots:
     void toolBoxAnalysisClusterabilitySelectChanged(int);
     void toolBoxLayoutByIndexButtonPressed();
 
+    QList<QGraphicsItem *> selectedNodes();
+
 protected:
     void resizeEvent( QResizeEvent * );
     void closeEvent( QCloseEvent* ce );
@@ -311,11 +342,20 @@ signals:
 
 private:
 
-
     QGraphicsScene *scene;
 
     FilterEdgesByWeightDialog m_filterEdgesByWeightDialog;
     WebCrawlerDialog m_WebCrawlerDialog;
+
+    NodeEditDialog *m_nodeEditDialog;
+
+    RandErdosRenyiDialog *m_randErdosRenyiDialog;
+    RandSmallWorldDialog *m_randSmallWorldDialog;
+    RandScaleFreeDialog *m_randScaleFreeDialog;
+
+    PreviewForm *previewForm;
+    QList<QTextCodec *> codecs;
+    QString userSelectedCodecName;
     DataSetSelectDialog m_datasetSelectDialog;
     Graph activeGraph;
     QPrinter *printer;
@@ -326,23 +366,22 @@ private:
     QProgressDialog *progressDialog;
 
     Node *clickedJim;
-    Node *markedNode;
-    Edge *clickedLink;
+    Edge *clickedEdge;
 
     QMenu *importSubMenu, *exportSubMenu, *editMenu, *statMenu,  *helpMenu;
-    QMenu *optionsMenu, *colorOptionsMenu, *linkOptionsMenu, *nodeOptionsMenu, *viewOptionsMenu;
-    QMenu *editNodeMenu, *editLinkMenu, *centrlMenu, *layoutMenu;
+    QMenu *optionsMenu, *colorOptionsMenu, *edgeOptionsMenu, *nodeOptionsMenu, *viewOptionsMenu;
+    QMenu *editNodeMenu, *editEdgeMenu, *centrlMenu, *layoutMenu;
     QMenu *networkMenu, *randomNetworkMenu, *filterMenu;
     QMenu *randomLayoutMenu, *circleLayoutMenu, *levelLayoutMenu, *physicalLayoutMenu;
     QMenu *colorationMenu;
-    QCheckBox *moveSpringEmbedderBx, *moveFruchtermanBx, *moveKamandaBx,
+    QCheckBox *layoutEadesBx, *layoutFruchtermanBx, *layoutKamandaBx,
     *nodeSizesByOutDegreeBx,*nodeSizesByInDegreeBx,
     *layoutGuidesBx;
     QComboBox *toolBoxAnalysisGeodesicsSelect,*toolBoxAnalysisConnectivitySelect,
             *toolBoxAnalysisProminenceSelect, *toolBoxAnalysisClusterabilitySelect;
     QComboBox *toolBoxLayoutByIndexSelect, *toolBoxLayoutByIndexTypeSelect;
 
-    QPushButton *addNodeBt, *addLinkBt, *removeNodeBt, *removeLinkBt,
+    QPushButton *addNodeBt, *addEdgeBt, *removeNodeBt, *removeEdgeBt,
     *toolBoxLayoutByIndexButton;
 
     QSpinBox *rotateSpinBox ;
@@ -353,20 +392,21 @@ private:
 
     QAction *viewNetworkFileAct, *openTextEditorAct, *viewSociomatrixAct, *recreateDataSetAct;
 
-    QAction *createUniformRandomNetworkAct, *createGaussianRandomNetworkAct, *createLatticeNetworkAct;
-    QAction *createSmallWorldRandomNetworkAct, *createSameDegreeRandomNetworkAct;
+    QAction *createErdosRenyiRandomNetworkAct, *createGaussianRandomNetworkAct;
+    QAction *createLatticeNetworkAct, *createScaleFreeRandomNetworkAct;
+    QAction *createSmallWorldRandomNetworkAct, *createRegularRandomNetworkAct;
+
     QAction *displayNodeNumbersAct, *displayNodeLabelsAct, *displayNumbersInsideNodesAct;
-    QAction *findNodeAct,*addNodeAct, *addLinkAct, *removeNodeAct, *removeLinkAct;
-    QAction *changeNumbersSizeAct, *changeNodeLabelAct, *changeNodeColorAct, *changeNodeValueAct, *changeNodeSizeAct;
+    QAction *selectNoneAct, *selectAllAct;
+    QAction *findNodeAct,*addNodeAct, *addEdgeAct, *removeNodeAct, *propertiesNodeAct, *removeEdgeAct;
+    QAction *changeNumbersSizeAct;
     QAction *changeLabelsSizeAct, *changeAllNodesSizeAct, *changeAllNodesShapeAct;
-    QAction *changeNodeBoxAct, *changeNodeCircleAct, *changeNodeTriangleAct, *changeNodeDiamondAct,
-            *changeNodeEllipseAct;
-    QAction *changeLinkLabelAct, *changeLinkColorAct, *changeLinkWeightAct;
-    QAction *filterNodesAct, *filterIsolateNodesAct, *filterEdgesAct, *transformNodes2LinksAct, *symmetrizeAct;
-    QAction *changeBackColorAct, *changeAllNodesColorAct, *changeAllLinksColorAct, *changeAllNumbersColorAct,
+    QAction *changeEdgeLabelAct, *changeEdgeColorAct, *changeEdgeWeightAct;
+    QAction *filterNodesAct, *filterIsolateNodesAct, *filterEdgesAct, *transformNodes2EdgesAct, *symmetrizeAct;
+    QAction *changeBackColorAct, *changeAllNodesColorAct, *changeAllEdgesColorAct, *changeAllNumbersColorAct,
             *changeAllLabelsColorAct;
-    QAction *drawLinksWeightsAct, *displayLinksWeightNumbersAct, *displayLinksAct, *displayLinksArrowsAct,
-            *drawLinksBezier;
+    QAction *drawEdgesWeightsAct, *displayEdgesWeightNumbersAct, *displayEdgesAct;
+    QAction *displayEdgesArrowsAct, *drawEdgesBezier,*considerEdgeWeightsAct;
     QAction *backgroundImageAct, *viewToolBar, *viewStatusBar, *helpAboutApp, *helpAboutQt, *helpApp, *tipsApp;
     QAction *antialiasingAct;
     QAction *webCrawlerAct;
@@ -376,7 +416,7 @@ private:
     QAction *walksAct,*totalWalksAct, *reachabilityMatrixAct, *connectednessAct;
     QAction *cliquesAct, *clusteringCoefAct, *triadCensusAct, *invertAdjMatrixAct;
     QAction *cDegreeAct, *cInDegreeAct, *cClosenessAct, *cInfluenceRangeClosenessAct,
-            *cBetweenessAct, *cInformationAct, *cPageRankAct, *cStressAct,
+            *cBetweennessAct, *cInformationAct, *cPageRankAct, *cStressAct,
             *cPowerAct, *cEccentAct, *cProximityPrestigeAct;
     QAction *randLayoutAct, *randCircleLayoutAct, *clearGuidesAct;
     QAction *layoutCircular_DC_Act, *layoutCircular_DP_Act,
@@ -394,26 +434,31 @@ private:
     QAction *nextRelationAct, *prevRelationAct, *addRelationAct;
 
     QString fileName, networkName, previous_fileName;
-    QString dataDir;
+    QString dataDir, lastUsedDirPath;
     QStringList fileNameNoPath, fortuneCookie, rgbValues;
     QStringList tempFileNameNoPath, colorList, tips;
     int statusBarDuration,  minDuration, progressCounter;
     int maxNodes;
     int initNodeSize, labelDistance, numberDistance,initNumberSize, initLabelSize;
-    int totalLinks, fortuneCookiesCounter,  tipsCounter;
+    int fortuneCookiesCounter,  tipsCounter;
     //QString VERSION;
-    bool pajekFileLoaded, adjacencyFileLoaded, dotFileLoaded, graphMLFileLoaded, fileLoaded;
+    bool pajekFileLoaded, adjacencyFileLoaded, dotFileLoaded, graphMLFileLoaded;
+    bool fileLoaded, checkSelectFileType;
     int fileFormat;
     bool networkModified;
-    bool bezier,  linkClicked, nodeClicked, markedNodeExists, showProgressBar, firstTime;
-    QString initLinkColor, initNumberColor,  initNodeShape, initLabelColor;
+    bool bezier,  edgeClicked, nodeClicked, markedNodesExist, showProgressBar, firstTime;
+    bool considerWeights, inverseWeights, askedAboutWeights;
+    QString initFileCodec;
+    QString initEdgeColor, initNumberColor,  initNodeShape, initLabelColor;
     QColor initBackgroundColor;
     QPointF cursorPosGW;	//Carries the position of the cursor in graphicsWidget coordinates
-    QLCDNumber  *inLinksLCD, *outLinksLCD , *selectedNodeLCD, *clucofLCD;
+    QLCDNumber  *inDegreeLCD, *outDegreeLCD , *selectedNodeLCD, *clucofLCD;
     QLCDNumber *nodesLCD, *edgesLCD, *densityLCD;
 
     QDateTime actualDateTime, actualDate, actualTime;
     QTime eTime;     //used  to time algorithms.
+
+
 };
 #endif 
 
