@@ -1,11 +1,11 @@
 /***************************************************************************
- SocNetV: Social Networks Visualizer 
- version: 1.4
+ SocNetV: Social Network Visualizer 
+ version: 1.9
  Written in Qt
 
                         node.cpp  -  description
                         -------------------
-    copyright            : (C) 2005-2014 by Dimitris B. Kalamaras
+    copyright            : (C) 2005-2015 by Dimitris B. Kalamaras
     email                : dimitris.kalamaras@gmail.com
  ***************************************************************************/
 
@@ -84,17 +84,21 @@ Node::Node( GraphicsWidget* gw, int num, int size,
 void Node::setColor(QString str) {
 	prepareGeometryChange();
 	m_col=QColor(str);
+    update();
 }
 
 /** 
 	Used by MW::slotFindNode() 
 */
 void Node::setColor(QColor color){
+    prepareGeometryChange();
 	m_col=color;
+    m_col_str = m_col.name();
+    update();
 }
 
 
-QString Node::color () { 
+QString Node::color() {
 	return m_col_str;
 }
 
@@ -125,11 +129,12 @@ int Node::size(){
 
 
 /**  Called every time the user needs to change the shape of an node. */
-void Node::setShape(QString shape) {
+void Node::setShape(const QString shape) {
 	qDebug("Node: setShape()");
 	prepareGeometryChange();
 	m_shape=shape;
 	qDebug ("Node: setShape(): node is at x=%f and y=%f", x(), y());
+    update();
 }
 
 
@@ -163,7 +168,7 @@ QRectF Node::boundingRect() const {
 void Node::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *) {
 //	painter->setClipRect( option->exposedRect );
 
-	//if the node is being dragged aroung, darken it!
+    //if the node is being dragged around, darken it!
 	if (option->state & QStyle::State_Selected) {
 		//qDebug()<< " node : selected ";
 		painter->setBrush(m_col.dark(150));
@@ -319,13 +324,14 @@ QVariant Node::itemChange(GraphicsItemChange change, const QVariant &value) {
 
 /** handles the events of a click on a node */
 void Node::mousePressEvent(QGraphicsSceneMouseEvent *event) {  
-	qDebug("Node: >> pressEvent() emitting nodeClicked");
+    qDebug() << "Node::mousePressEvent() "
+                << " set selected and emitting nodeClicked";
+    this->setSelected(true);
 //	emit nodeClicked(this);
 	graphicsWidget->nodeClicked(this);
 	if ( event->button()==Qt::LeftButton ) {
-		qDebug("Node: pressEvent() left click > startNodeMovement");
-//		emit startNodeMovement(0);
-//		graphicsWidget->startNodeMovement(0);
+        qDebug("Node::mousePressEvent() left click ");
+
 	}
 	if ( event->button()==Qt::RightButton ) {
 		qDebug("Node: Right-click on node, at %i, %i", event->screenPos().x(), event->screenPos().y()); 
@@ -340,6 +346,7 @@ void Node::mousePressEvent(QGraphicsSceneMouseEvent *event) {
 //		emit startEdge(this);
 		graphicsWidget->startEdge(this);
 	}
+
 }
 
 
