@@ -1,11 +1,11 @@
 /***************************************************************************
- SocNetV: Social Networks Visualizer
- version: 1.4
+ SocNetV: Social Network Visualizer
+ version: 1.9
  Written in Qt
  
                          node.h  -  description
                           -------------------
-    copyright            : (C) 2005-2014 by Dimitris B. Kalamaras
+    copyright            : (C) 2005-2015 by Dimitris B. Kalamaras
     email                : dimitris.kalamaras@gmail.com
  ***************************************************************************/
 
@@ -73,14 +73,14 @@ public:
 	void setSize(int);
 	int size();
 
-	void setShape (QString);
+    void setShape (const QString);
 	QString nodeShape() {return m_shape;}
 
 	void setColor(QString str);
 	void setColor(QColor color);
 	QString color ();
 	
-	void setLabelText ( QString label) ;		// Used by MW:: slotChangeNodeLabel()
+    void setLabelText ( QString label) ;
 	QString labelText () ;						// Used by GW:: hasNode()
 	NodeLabel* label();
 	void addLabel (NodeLabel* gfxLabel  ) ;
@@ -106,7 +106,6 @@ protected:
 	void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
 signals: 
 	void nodeClicked(Node*);
-	void startNodeMovement(int);	
 	void openNodeContextMenu();
 	void startEdge(Node *);
 	void adjustOutEdge();
