@@ -1,11 +1,11 @@
 /***************************************************************************
- SocNetV: Social Networks Visualizer 
- version: 1.4
+ SocNetV: Social Network Visualizer 
+ version: 1.9
  Written in Qt
  
                          parser.h  -  description
                              -------------------
-    copyright            : (C) 2005-2014 by Dimitris B. Kalamaras
+    copyright            : (C) 2005-2015 by Dimitris B. Kalamaras
     email                : dimitris.kalamaras@gmail.com
  ***************************************************************************/
 
@@ -30,22 +30,29 @@
 using namespace std;
 
 #include <QThread>
+#include <QHash>
+#include <QStringList>
 #include <QPointF>
-#include <QMutex>
 #include <QObject>
-#include <QtXml>	
 #include <QMultiMap>
 
+class QXmlStreamReader;
+class QXmlStreamAttributes;
 /** 	
 	Main class for network file parsing and loading
 	Currently, it supports Pajek, Adjacency, Graphviz, GraphML
 */
-class Parser :  public QThread {
+class Parser :  public QObject {
 	Q_OBJECT
 public:
 	
-	bool load(QString fn, int iNS, QString iNC, QString iNSh, QString iNNC,
-		 int iNNS, QString iNLC, int iNLS , QString iEC, int w, int h, int format, int sm_mode);
+    Parser(const QString fn, const QString codec, const int iNS,
+              const QString iNC, const QString iNSh, const QString iNNC,
+              const int iNNS, const QString iNLC, const int iNLS ,
+              const QString iEC, const int w, const int h, const int format,
+              const int sm_mode);
+    ~Parser();
+    bool run();
 	bool loadPajek();
 	bool loadAdjacency();
 	bool loadDot();
@@ -57,7 +64,9 @@ public:
 	bool loadWeighedList();
 	bool loadTwoModeSociomatrix();
 
-	void dotProperties(QString str, float &, QString &label, QString &shape, QString &color, QString &fontName, QString &fontColor );
+    void dotProperties(QString str, float &, QString &label,
+                       QString &shape, QString &color, QString &fontName,
+                       QString &fontColor );
 	void readGraphML (QXmlStreamReader &);
 	void readGraphMLElementGraph(QXmlStreamReader &);
 	void readGraphMLElementNode (QXmlStreamReader &);
@@ -89,19 +98,19 @@ signals:
 	void createEdge (int, int, float, QString, int, bool, bool);
 	void fileType(int, QString, int, int, bool);
 	void removeDummyNode (int);
+    void finished(QString);
 	
 protected:
-	void run();
+
 private: 
-	QMutex mutex;
 	QHash<QString, int> nodeNumber;
 	QHash<QString, QString> keyFor, keyName, keyType, keyDefaultValue ;
     QHash<QString, QString> edgesMissingNodesHash;
     QStringList edgeMissingNodesList,edgeMissingNodesListData;
 	QMultiMap<int, int> firstModeMultiMap, secondModeMultiMap;
 	QXmlStreamReader *xml;
-    QString fileName, networkName, initNodeColor, initEdgeColor, initNodeShape;
-    QString initNodeNumberColor, initNodeLabelColor;
+    QString fileName, userSelectedCodecName, networkName, initNodeColor;
+    QString initEdgeColor, initNodeShape, initNodeNumberColor, initNodeLabelColor;
     QString nodeColor, edgeColor, edgeType, nodeShape, nodeLabel, edgeLabel;
     QString nodeNumberColor, nodeLabelColor;
     QString key_id, key_value, key_name, key_what, key_type;
