@@ -1,11 +1,11 @@
 /****************************************************************************
-SocNetV: Social Networks Visualizer 
-version: 1.4
+SocNetV: Social Network Visualizer 
+version: 1.9
 Written in Qt
 
                                texteditor.cpp 
                              -------------------
-    copyright            : (C) 2005-2014 by Dimitris B. Kalamaras
+    copyright            : (C) 2005-2015 by Dimitris B. Kalamaras
     email                : dimitris.kalamaras@gmail.com
 *****************************************************************************/
 
@@ -28,7 +28,7 @@ Written in Qt
 #include <QtWidgets>
 #include "texteditor.h"
 
-TextEditor::TextEditor(const QString &fileName)
+TextEditor::TextEditor(const QString &fileName, QWidget *parent) : QMainWindow(parent)
 {
 	qDebug("TextEditor()");
 	textEdit = new QTextEdit;
@@ -250,6 +250,7 @@ void TextEditor::loadFile(const QString &fileName)
 
     setCurrentFile(fileName);
     statusBar()->showMessage(tr("File loaded"), 2000);
+    file.close();
 }
 
 bool TextEditor::saveFile(const QString &fileName)
@@ -263,13 +264,15 @@ bool TextEditor::saveFile(const QString &fileName)
         return false;
     }
 
-    QTextStream out(&file);
+    QTextStream outText(&file);
+    outText.setCodec("UTF-8");
     QApplication::setOverrideCursor(Qt::WaitCursor);
-    out << textEdit->toPlainText();
+    outText << textEdit->toPlainText();
     QApplication::restoreOverrideCursor();
 
     setCurrentFile(fileName);
     statusBar()->showMessage(tr("File saved"), 2000);
+    file.close();
     return true;
 }
 
