@@ -1,11 +1,11 @@
 /****************************************************************************
-SocNetV: Social Networks Visualizer 
-version: 1.4
+SocNetV: Social Network Visualizer 
+version: 1.9
 Written in Qt
 
 			       texteditor.h
                              -------------------
-    copyright            : (C) 2005-2014 by Dimitris B. Kalamaras
+    copyright            : (C) 2005-2015 by Dimitris B. Kalamaras
     email                : dimitris.kalamaras@gmail.com
 *****************************************************************************/
 
@@ -38,7 +38,7 @@ class TextEditor : public QMainWindow
     Q_OBJECT
 
 public:
-	TextEditor(const QString &fileName);
+    TextEditor(const QString &fileName , QWidget *parent=0 );
 
 protected:
 	void closeEvent(QCloseEvent *event);
