@@ -1,11 +1,11 @@
 /***************************************************************************
- SocNetV: Social Networks Visualizer 
- version: 1.4
+ SocNetV: Social Network Visualizer 
+ version: 1.9
  Written in Qt
  
                          vertex.cpp  -  description
                              -------------------
-    copyright            : (C) 2005-2014 by Dimitris B. Kalamaras
+    copyright            : (C) 2005-2015 by Dimitris B. Kalamaras
     email                : dimitris.kalamaras@gmail.com
  ***************************************************************************/
 
@@ -28,17 +28,24 @@
 
 
 #include <QtDebug>		//used for qDebug messages
-#include <QPointF>
 #include "graph.h"
 
-Vertex::Vertex(	Graph* parent, 
-				int v1,  int val, int size, QString color, 
-				QString numColor, int numSize, 
-				QString label, QString labelColor, int labelSize,
-				QPointF p, 
-				QString shape): parentGraph (parent)  
+Vertex::Vertex(Graph* parent,
+               const long &name,
+               const int &val,
+               const int &relation,
+               const int &size,
+               const QString &color,
+               const QString &numColor,
+               const int &numSize,
+               const QString &label,
+               const QString &labelColor,
+               const int &labelSize,
+               const QPointF &p,
+               const QString &shape): parentGraph (parent)
 { 
-	m_name=v1; 
+    qDebug() << "Vertex::Vertex() - "<<  name << " setting values";
+    m_name=name;
 	m_value=val;
 	m_size=size;
 	m_color=color;
@@ -52,10 +59,12 @@ Vertex::Vertex(	Graph* parent,
 	m_y=p.y();
 	//FIXME outLinkColors list need update when we remove vertices/edges
 //	outLinkColors=new  QString[1500];
-	//Q_CHECK_PTR(outLinkColors);
-	outLinkColors.reserve(1600);	
-    m_outLinksCounter=0;
-    m_inLinksCounter=0;
+    //Q_CHECK_PTR(outLinkColors);
+    //outLinkColors.reserve(2000);
+    m_outEdges.reserve(100);
+    m_inEdges.reserve(100);
+    m_outEdgesCounter=0;
+    m_inEdgesCounter=0;
     m_outDegree=0;
     m_inDegree=0;
     m_localDegree=0;
@@ -63,33 +72,31 @@ Vertex::Vertex(	Graph* parent,
     m_DC=0; m_SDC=0; m_DP=0; m_SDP=0; m_CC=0; m_SCC=0; m_BC=0; m_SBC=0;
     m_SC=0; m_SSC=0; m_IRCC=0; m_SIRCC=0;
     m_CLC=0; m_hasCLC=false;
-    m_curRelation=0;
-    m_inLinked=false;
-    m_outLinked=false;
+    m_curRelation=relation;
     m_reciprocalLinked=false;
     m_enabled = true;
-//    m_outLinks.reserve(1000);
+
     connect (this, SIGNAL (setEdgeVisibility ( int, int, int, bool) ),
              parent, SLOT (slotSetEdgeVisibility (int, int, int, bool)) );
 
 }
 
-Vertex::Vertex(int v1) { 
-	m_name=v1; 
+// constructor with default values
+Vertex::Vertex(const long int &name) {
+    qDebug() << "Vertex::Vertex() - "<<  name << " using default values";
+    m_name=name;
 	m_value=1;
 	m_size=9;
 	m_color="black";
 	m_label="";
 	m_labelColor="black";
 	m_shape="circle";
-    m_outLinksCounter=0;
-    m_inLinksCounter=0;
+    m_outEdgesCounter=0;
+    m_inEdgesCounter=0;
     m_Eccentricity=0;
     m_DC=0; m_SDC=0; m_DP=0; m_SDP=0; m_CC=0; m_SCC=0; m_BC=0; m_SBC=0;
     m_IRCC=0; m_SIRCC=0; m_SC=0; m_SSC=0;
     m_curRelation=0;
-    m_inLinked=false;
-    m_outLinked=false;
     m_reciprocalLinked=false;
 }
 
@@ -99,8 +106,8 @@ Vertex::Vertex(int v1) {
 * @param relation
 */
 void Vertex::changeRelation(int relation) {
-    qDebug() << "Vertice::changeRelation() to relation " << relation
-                << " from current relation " << m_curRelation;
+    qDebug() << "Vertex::changeRelation() - Current: " << m_curRelation
+                << " new: " << relation;
     // first make false all edges of current relation
     filterEdgesByRelation(m_curRelation, false);
     // then make true all edges of new relation
@@ -110,32 +117,41 @@ void Vertex::changeRelation(int relation) {
 }
 
 
+QString Vertex::colorToPajek(){
+    if (m_color.startsWith("#")) {
+        return  ("RGB"+m_color.right( m_color.size()-1 )).toUpper()  ;
+    }
+    return m_color;
+}
+
+
+
 /**
- * @brief Vertex::addLinkTo
+ * @brief Vertex::addEdgeTo
+ * Adds an outLink to target with weight w
  * @param target
  * @param weight
  */
-void Vertex::addLinkTo (long int target, float weight) {
-    qDebug() <<"Vertex::addLinkTo() - new link "
-            << name() << " -> "<< target << " weight "<< weight
+void Vertex::addEdgeTo (const long &v2, const float &weight) {
+    qDebug() <<"Vertex::addEdgeTo() - new link "
+            << name() << " -> "<< v2 << " weight "<< weight
                << " relation " << m_curRelation;
-    // do not use [] operator. It silently creates an item if key do not exist
-    m_outLinks.insertMulti(
-                target, rel_w_bool(m_curRelation, pair_f_b(weight, true) ) );
-    m_outLinksCounter++;
+    // do not use [] operator - silently creates an item if key do not exist
+    m_outEdges.insertMulti(
+                v2, rel_w_bool(m_curRelation, pair_f_b(weight, true) ) );
 }
 
 
 /**
- * @brief Vertex::setOutLinkEnabled
+ * @brief Vertex::setOutEdgeEnabled
  * @param target
  * @param status
  */
-void Vertex::setOutLinkEnabled (long int target, bool status){
-    qDebug () << "Vertex::setOutLinkEnabled - set outLink to " << target
+void Vertex::setOutEdgeEnabled (long int target, bool status){
+    qDebug () << "Vertex::setOutEdgeEnabled - set outLink to " << target
               << " as " << status
                  << ". Finding outLink...";
-    QMutableHashIterator < int, rel_w_bool > it1 (m_outLinks);
+    QMutableHashIterator < int, rel_w_bool > it1 (m_outEdges);
     int linkTarget=0;
     float weight =0;
     int relation = 0;
@@ -162,70 +178,67 @@ void Vertex::setOutLinkEnabled (long int target, bool status){
 
 
 /**
- * @brief Vertex::addLinkFrom
+ * @brief Vertex::addEdgeFrom
  * @param source
  * @param weight
  */
-void Vertex::addLinkFrom (long int source, float weight) {
-//    qDebug() <<"Vertex: "<< name() << " addLinkFrom() "<< source;
-    m_inLinks.insertMulti(
-                source, rel_w_bool (m_curRelation, pair_f_b(weight, true) ) );
-    m_inLinksCounter++;
+void Vertex::addEdgeFrom (const long int &v1, const float &weight) {
+//    qDebug() <<"Vertex: "<< name() << " addEdgeFrom() "<< source;
+    m_inEdges.insertMulti(
+                v1, rel_w_bool (m_curRelation, pair_f_b(weight, true) ) );
 }
 
-void Vertex::changeLinkWeightTo(long int target, float weight){
-    qDebug() << "Vertex::changeEdgeWeightTo " << target;
-    qDebug() << " *** m_outLinks.count " <<
-                m_outLinks.count();
+
+void Vertex::changeOutEdgeWeight(long int target, float weight){
+    qDebug() << "Vertex::changeEdgeWeightTo " << target << " weight " << weight ;
+    qDebug() << " *** m_outEdges.count " <<
+                m_outEdges.count();
     qDebug() << "first find and remove old relation-weight pair" ;
-    H_edges::iterator it1=m_outLinks.find(target);
-    while (it1 != m_outLinks.end() ) {
+    H_edges::iterator it1=m_outEdges.find(target);
+    while (it1 != m_outEdges.end() ) {
         if ( it1.key() == target && it1.value().first == m_curRelation ) {
-            it1=m_outLinks.erase(it1);
+            it1=m_outEdges.erase(it1);
         }
         else {
             ++it1;
         }
     }
-    qDebug() << " *** m_outLinks.count " <<
-                m_outLinks.count();
+    qDebug() << " *** m_outEdges.count " <<
+                m_outEdges.count();
     qDebug() << " create new relation-weight pair ";
-    m_outLinks.insertMulti(
+    m_outEdges.insertMulti(
                 target, rel_w_bool(m_curRelation, pair_f_b(weight, true) ) );
-    qDebug() << " *** m_outLinks.count " << m_outLinks.count();
+    qDebug() << " *** m_outEdges.count " << m_outEdges.count();
 }
 
 
 
 /**
- * @brief Vertex::removeLinkTo
- * finds and removes a link to vertice v2
+ * @brief Vertex::removeEdgeTo
+ * finds and removes a link to vertex v2
  * @param v2
  */
-void Vertex::removeLinkTo (long int v2) {
-    qDebug() << "Vertex: removeLinkTo() - vertex " << m_name
-             << " has " <<outLinks() << " out-links. Removing link to "<< v2 ;
+void Vertex::removeEdgeTo (long int v2) {
+    qDebug() << "Vertex: removeEdgeTo() - vertex " << m_name
+             << " has " <<outEdges() << " out-links. Removing link to "<< v2 ;
 
-    if (outLinks()>0) {
-        qDebug () << "checking all_outLinks";
-        H_edges::iterator it1=m_outLinks.find(v2);
-        while (it1 != m_outLinks.end() && it1.key() == v2 ) {
+    if (outEdges()>0) {
+        qDebug () << "checking all_outEdges";
+        H_edges::iterator it1=m_outEdges.find(v2);
+        while (it1 != m_outEdges.end() && it1.key() == v2 ) {
             if ( it1.value().first == m_curRelation ) {
                 qDebug() << " *** vertex " << m_name << " connected to "
                          << it1.key() << " relation " << it1.value().first
                          << " weight " << it1.value().second.first
                          << " enabled ? " << it1.value().second.second;
-                qDebug() << " *** erasing outEdge from m_outLinks ";
-                it1=m_outLinks.erase(it1);
+                qDebug() << " *** erasing outEdge from m_outEdges ";
+                it1=m_outEdges.erase(it1);
             }
             else {
                 ++it1;
             }
         }
-
-        m_outLinksCounter--;
-
-        qDebug() << "Vertex: vertex " <<  m_name << " now has " <<  outLinks() << " out-edges";
+        qDebug() << "Vertex: vertex " <<  m_name << " now has " <<  outEdges() << " out-edges";
 	}
 	else {
 		qDebug() << "Vertex: vertex " <<  m_name << " has no edges" ;
@@ -234,33 +247,31 @@ void Vertex::removeLinkTo (long int v2) {
 
 
 /**
- * @brief Vertex::removeLinkFrom
+ * @brief Vertex::removeEdgeFrom
  * @param v2
  */
-void Vertex::removeLinkFrom(long int v2){
-    qDebug() << "Vertex: removeLinkFrom() vertex " << m_name
-             << " has " <<  inLinks() << "  in-edges. RemovingEdgeFrom " << v2 ;
+void Vertex::removeEdgeFrom(long int v2){
+    qDebug() << "Vertex: removeEdgeFrom() vertex " << m_name
+             << " has " <<  inEdges() << "  in-edges. RemovingEdgeFrom " << v2 ;
 
-    if (inLinks()>0) {
-        qDebug () << "checking all_inLinks";
-        H_edges::iterator it=m_inLinks.find(v2);
-        while (it != m_inLinks.end() ) {
+    if (inEdges()>0) {
+        qDebug () << "checking all_inEdges";
+        H_edges::iterator it=m_inEdges.find(v2);
+        while (it != m_inEdges.end() ) {
             if ( it.key() == v2 && it.value().first == m_curRelation ) {
                 qDebug() << " *** vertex " << m_name << " connected from  "
                          << it.key() << " relation " << it.value().first
                          << " weight " << it.value().second.first
                          << " enabled ? " << it.value().second.second;
-                qDebug() << " *** erasing inEdge from m_inLinks ";
-                it=m_inLinks.erase(it);
+                qDebug() << " *** erasing inEdge from m_inEdges ";
+                it=m_inEdges.erase(it);
             }
             else {
                 ++it;
             }
         }
-        m_inLinksCounter--;
-
         qDebug() << "Vertex: vertex " << m_name << " now has "
-                 << inLinks() << " in-links"  ;
+                 << inEdges() << " in-links"  ;
 	}
 	else {
 		qDebug() << "Vertex: vertex " << m_name << " has no edges";
@@ -280,7 +291,7 @@ void Vertex::filterEdgesByWeight(float m_threshold, bool overThreshold){
 	qDebug() << "Vertex::filterEdgesByWeight of vertex " << this->m_name;
 	int target=0;
     float weight=0;
-    QMutableHashIterator < int, rel_w_bool > it (m_outLinks);
+    QMutableHashIterator < int, rel_w_bool > it (m_outEdges);
     while ( it.hasNext()) {
         it.next();
         if ( it.value().first == m_curRelation ) {
@@ -329,12 +340,12 @@ void Vertex::filterEdgesByWeight(float m_threshold, bool overThreshold){
  * @param relation
  */
 void Vertex::filterEdgesByRelation(int relation, bool status ){
-    qDebug() << "Vertex::filterEdgesByRelation of vertex " << this->m_name
-                << " relation " << relation << " to " << status;
+    qDebug() << "Vertex::filterEdgesByRelation() - Vertex " << this->m_name
+                << " filtering edges of relation " << relation << " to " << status;
     int target=0;
     float weight =0;
     int edgeRelation=0;
-    QMutableHashIterator < int, rel_w_bool > it1 (m_outLinks);
+    QMutableHashIterator < int, rel_w_bool > it1 (m_outEdges);
     while ( it1.hasNext()) {
         it1.next();
         edgeRelation = it1.value().first;
@@ -356,96 +367,194 @@ void Vertex::filterEdgesByRelation(int relation, bool status ){
 
 
 /**
- * @brief Vertex::outLinks
+ * @brief Vertex::outEdges
  * Returns the number of active outbound arcs, aka the number of
- * outLinks, from this vertex for the current relation
+ * outEdges, from this vertex for the current relation
  * @return long int
  */
-long int Vertex::outLinks() {
-    m_outLinksCounter = 0;
+long int Vertex::outEdges() {
+    m_outEdgesCounter = 0;
     int relation=0;
     bool edgeStatus = false;
-    H_edges::const_iterator it1=m_outLinks.constBegin();
-    while (it1 != m_outLinks.constEnd() ) {
+    H_edges::const_iterator it1=m_outEdges.constBegin();
+    while (it1 != m_outEdges.constEnd() ) {
         relation = it1.value().first;
         if ( relation == m_curRelation ) {
             edgeStatus=it1.value().second.second;
             if ( edgeStatus == true) {
-                m_outLinksCounter++;
+                m_outEdgesCounter++;
             }
         }
         ++it1;
     }
-    return m_outLinksCounter;
+    return m_outEdgesCounter;
 }
 
 /**
- * @brief Vertex::returnEnabledOutLinks
- * Returns a qhash of all enabled outLinks in the active relation
+ * @brief Vertex::outEdgesConst
+ * Returns the number of active outbound arcs
+ * Needs to have outEdges called before the call to this method
+ * @return long int
+ */
+long int Vertex::outEdgesConst() const {
+    return m_outEdgesCounter;
+}
+
+
+/**
+ * @brief Vertex::returnEnabledOutEdges
+ * Returns a qhash of all enabled outEdges in the active relation
  * @return  QHash<int,float>*
  */
-QHash<int,float>* Vertex::returnEnabledOutLinks(){
-    qDebug() << " Vertex:: returnEnabledOutLinks()";
-    QHash<int,float> *enabledOutLinks = new QHash<int,float>;
+QHash<int,float>* Vertex::returnEnabledOutEdges(){
+    //qDebug() << " Vertex::returnEnabledOutEdges() vertex " << this->name();
+    QHash<int,float> *enabledOutEdges = new QHash<int,float>;
     float m_weight=0;
     int relation = 0;
     bool edgeStatus=false;
-    H_edges::const_iterator it1=m_outLinks.constBegin();
-    while (it1 != m_outLinks.constEnd() ) {
+    H_edges::const_iterator it1=m_outEdges.constBegin();
+    while (it1 != m_outEdges.constEnd() ) {
         relation = it1.value().first;
         if ( relation == m_curRelation ) {
             edgeStatus=it1.value().second.second;
             if ( edgeStatus == true) {
                 m_weight=it1.value().second.first;
-                enabledOutLinks->insert(it1.key(), m_weight);
+                enabledOutEdges->insert(it1.key(), m_weight);
+//                qDebug() <<  " Vertex::returnEnabledOutEdges() count:"
+//                             << enabledOutEdges->count();
             }
         }
         ++it1;
     }
-    return enabledOutLinks;
+//    qDebug() << " Vertex::returnEnabledOutEdges() vertex " << this->name()
+//                << " outEdges count:"
+//                 << enabledOutEdges->count();
+    return enabledOutEdges;
 }
 
 
+
 /**
- * @brief Vertex::inLinks
- * Returns the number of active inbound arcs, aka the number of
- * inLinks, to this vertex for the current relation
- * @return long int
+ * @brief Vertex::allReciprocalEdges
+ * Returns a qhash of all reciprocal edges to neighbors in the active relation
+ * @return  QHash<int,float>*
  */
-long int Vertex::inLinks() {
-    m_inLinksCounter = 0;
-    int relation=0;
-    bool edgeStatus = false;
-    H_edges::const_iterator it1=m_inLinks.constBegin();
-    while (it1 != m_inLinks.constEnd() ) {
+QHash<int,float>* Vertex::returnReciprocalEdges(){
+//    qDebug() << "Vertex::returnReciprocalEdges() - vertex " << this->name();
+    QHash<int,float> *reciprocalEdges = new QHash<int,float>;
+    float m_weight=0;
+    int relation = 0;
+    bool edgeStatus=false;
+    H_edges::const_iterator it1=m_outEdges.constBegin();
+    while (it1 != m_outEdges.constEnd() ) {
         relation = it1.value().first;
         if ( relation == m_curRelation ) {
             edgeStatus=it1.value().second.second;
             if ( edgeStatus == true) {
-                m_inLinksCounter++;
+                m_weight=it1.value().second.first;
+                if (this->hasEdgeFrom (it1.key()) )
+                    reciprocalEdges->insertMulti(it1.key(), m_weight);
             }
         }
         ++it1;
     }
-    return m_inLinksCounter;
+
+    qDebug() << "Vertex::returnReciprocalEdges() - vertex " << this->name()
+             <<  " = "
+              << reciprocalEdges->count();
+
+    return reciprocalEdges;
 }
 
 
 
+/**
+ * @brief Vertex::neighborhood
+ * Returns a qhash of all neighbors in the active relation
+ * @return  QHash<int,float>*
+ */
+//QHash<int,float>* Vertex::neighborhood(){
+//    qDebug() << " Vertex::neighborhood() vertex " << this->name();
+//    QHash<int,float> *neighbors = new QHash<int,float>;
+//    float m_weight=0;
+//    int relation = 0;
+//    bool edgeStatus=false;
+//    H_edges::const_iterator it1=m_outEdges.constBegin();
+//    while (it1 != m_outEdges.constEnd() ) {
+//        relation = it1.value().first;
+//        if ( relation == m_curRelation ) {
+//            edgeStatus=it1.value().second.second;
+//            if ( edgeStatus == true) {
+//                m_weight=it1.value().second.first;
+//                if (this->hasEdgeFrom (it1.key()) )
+//                    neighbors->insertMulti(it1.key(), m_weight);
+//                qDebug() <<  " Vertex::returnReciprocalEdges() count:"
+//                             << reciprocalEdges->count();
+//            }
+//        }
+//        ++it1;
+//    }
+
+
+//    qDebug() <<  " Vertex::returnReciprocalEdges() total "
+//                 << reciprocalEdges->count();
+
+//    qDebug() <<  " Vertex::returnReciprocalEdges() localDegree "
+//                 << this->localDegree();
+//    return reciprocalEdges;
+//}
+
+
+/**
+ * @brief Vertex::inEdges
+ * Returns the number of active inbound arcs, aka the number of
+ * inEdges, to this vertex for the current relation
+ * @return long int
+ */
+long int Vertex::inEdges() {
+    m_inEdgesCounter = 0;
+    int relation=0;
+    bool edgeStatus = false;
+    H_edges::const_iterator it1=m_inEdges.constBegin();
+    while (it1 != m_inEdges.constEnd() ) {
+        relation = it1.value().first;
+        if ( relation == m_curRelation ) {
+            edgeStatus=it1.value().second.second;
+            if ( edgeStatus == true) {
+                m_inEdgesCounter++;
+            }
+        }
+        ++it1;
+    }
+    return m_inEdgesCounter;
+}
+
+
+/**
+ * @brief Vertex::inEdgesConst
+ * Returns the number of active inbound arcs
+ * Needs to have inEdges called before the call to this method
+ * @return long int
+ */
+long int Vertex::inEdgesConst() const {
+    return m_inEdgesCounter;
+}
+
+
 
 /**
  * @brief Vertex::outDegree
- * Returns the outDegree (the sum of all enabled outLinks weights) of this vertice
+ * Returns the outDegree (the sum of all enabled outEdges weights) of this vertex
  * @return long int
  */
 long int Vertex::outDegree() {
-    qDebug() << " Vertex:: outDegree()";
+    qDebug() << "Vertex::outDegree()";
     m_outDegree=0;
     float m_weight=0;
     int relation = 0;
     bool edgeStatus=false;
-    H_edges::const_iterator it1=m_outLinks.constBegin();
-    while (it1 != m_outLinks.constEnd() ) {
+    H_edges::const_iterator it1=m_outEdges.constBegin();
+    while (it1 != m_outEdges.constEnd() ) {
         relation = it1.value().first;
         if ( relation == m_curRelation ) {
             edgeStatus=it1.value().second.second;
@@ -459,21 +568,23 @@ long int Vertex::outDegree() {
     return m_outDegree;
 }
 
-
+long int Vertex::outDegreeConst() {
+    return m_outDegree;
+}
 
 /**
  * @brief Vertex::inDegree
- * Returns the inDegree (the sum of all enabled inLinks weights) of this vertice
+ * Returns the inDegree (the sum of all enabled inEdges weights) of this vertex
  * @return long int
  */
 long int Vertex::inDegree() {
-    qDebug() << " Vertex::inDegree()";
+    qDebug() << "Vertex::inDegree()";
     m_inDegree=0;
     float m_weight=0;
     int relation = 0;
     bool edgeStatus=false;
-    H_edges::const_iterator it1=m_inLinks.constBegin();
-    while (it1 != m_inLinks.constEnd() ) {
+    H_edges::const_iterator it1=m_inEdges.constBegin();
+    while (it1 != m_inEdges.constEnd() ) {
         relation = it1.value().first;
         if ( relation == m_curRelation ) {
             edgeStatus=it1.value().second.second;
@@ -489,6 +600,10 @@ long int Vertex::inDegree() {
 }
 
 
+long int Vertex::inDegreeConst() {
+    return m_inDegree;
+}
+
 
 /**
  	localDegree is the outDegree + inDegree minus the edges counted twice.
@@ -499,14 +614,14 @@ long int Vertex::localDegree(){
     bool edgeStatus=false;
     m_localDegree = (outDegree() + inDegree() );
 
-    H_edges::const_iterator it1=m_outLinks.constBegin();
-    while (it1 != m_outLinks.constEnd() ) {
+    H_edges::const_iterator it1=m_outEdges.constBegin();
+    while (it1 != m_outEdges.constEnd() ) {
         relation = it1.value().first;
         if ( relation == m_curRelation ) {
             edgeStatus=it1.value().second.second;
             if ( edgeStatus == true) {
                 v2=it1.key();
-                if (this->isLinkedFrom (v2) ) m_localDegree--;
+                if (this->hasEdgeFrom (v2) ) m_localDegree--;
             }
         }
         ++it1;
@@ -518,30 +633,30 @@ long int Vertex::localDegree(){
 
 
 /**
- * @brief Vertex::isLinkedTo
+ * @brief Vertex::hasEdgeTo
  * Checks if this vertex is outlinked to v2 and returns the weight of the link
  * only if the outLink is enabled.
  * @param v2
  * @return
  */
-float Vertex::isLinkedTo(long int v2){
-    qDebug()<< "Vertex::isLinkedTo()" ;
+float Vertex::hasEdgeTo(long int v2){
+//    qDebug()<< "Vertex::hasEdgeTo() " << name() << " -> " << v2 ;
     float m_weight=0;
     bool edgeStatus=false;
-    H_edges::iterator it1=m_outLinks.find(v2);
-    while (it1 != m_outLinks.end() && it1.key() == v2 ) {
+    H_edges::iterator it1=m_outEdges.find(v2);
+    while (it1 != m_outEdges.end() && it1.key() == v2 ) {
         if ( it1.value().first == m_curRelation ) {
             edgeStatus=it1.value().second.second;
             if ( edgeStatus == true) {
                 m_weight=it1.value().second.first;
-                qDebug()<< "***** Vertex::isLinkedTo() - relation "
-                           << it1.value().first
-                        <<" link "  <<  this->name()
-                        << " -> " << v2 << "exists, weight "<< m_weight;
+//                qDebug()<< "***** Vertex::hasEdgeTo() - relation "
+//                           << it1.value().first
+//                        <<" link "  <<  this->name()
+//                        << " -> " << v2 << "exists, weight "<< m_weight;
                 return m_weight;
             }
             else
-                qDebug()<< "Vertex::isLinkedTo() - relation "
+                qDebug()<< "Vertex::hasEdgeTo() - relation "
                            << it1.value().first
                         <<" link "  <<  this->name()
                         << " -> " << v2 << "exists, weight "<< m_weight
@@ -550,34 +665,34 @@ float Vertex::isLinkedTo(long int v2){
         }
         ++it1;
     }
-    qDebug()<< "Vertex::isLinkedTo() - INEXISTENT LINK IN RELATION " << m_curRelation;
+   // qDebug()<< "Vertex::hasEdgeTo() - INEXISTENT LINK IN RELATION " << m_curRelation;
 	return 0;
 }
 
 
 /**
- * @brief Vertex::isLinkedFrom
+ * @brief Vertex::hasEdgeFrom
  * Checks if this vertex is inLinked from v2 and returns the weight of the link
  * only if the inLink is enabled.
  * @param v2
  * @return
  */
-float Vertex::isLinkedFrom(long int v2){
-    qDebug()<< "Vertex::isLinkedFrom()" ;
+float Vertex::hasEdgeFrom(long int v2){
+    qDebug()<< "Vertex::hasEdgeFrom()" ;
     float m_weight=0;
     bool edgeStatus=false;
-    H_edges::iterator it1=m_inLinks.find(v2);
-    while (it1 != m_inLinks.end() && it1.key() == v2) {
+    H_edges::iterator it1=m_inEdges.find(v2);
+    while (it1 != m_inEdges.end() && it1.key() == v2) {
         if ( it1.value().first == m_curRelation ) {
             edgeStatus=it1.value().second.second;
             if ( edgeStatus == true) {
                 m_weight=it1.value().second.first;
-                qDebug()<< "Vertex::isLinkedFrom() - a ("  <<  this->name()
+                qDebug()<< "Vertex::hasEdgeFrom() - a ("  <<  this->name()
                         << ", " << v2 << ") = "<< m_weight;
                 return m_weight;
             }
             else
-                qDebug()<< "Vertex::isLinkedFrom() - a ("  <<  this->name()
+                qDebug()<< "Vertex::hasEdgeFrom() - a ("  <<  this->name()
                         << ", " << v2 << ") = "<< m_weight
                         << " but edgeStatus " << edgeStatus;
                 return 0;
@@ -585,148 +700,85 @@ float Vertex::isLinkedFrom(long int v2){
         }
         ++it1;
     }
-    qDebug()<< "Vertex::isLinkedFrom() - a ("  <<  this->name()  << ", " << v2 << ") = 0 ";
+    qDebug()<< "Vertex::hasEdgeFrom() - a ("  <<  this->name()  << ", " << v2 << ") = 0 ";
     return 0;
 }
 
 
-long int Vertex::name() {
-	return m_name;
+
+int Vertex::cliques (const int &size)
+{
+    int count = 0;
+    foreach (int value, m_cliques) {
+        if ( value == size ) {
+            count ++;
+        }
+    }
+    return count ;
+}
+
+bool Vertex::addClique (const QString &clique, const int &size) {
+    QStringList members = clique.split(",");
+    switch (size) {
+    case 2:
+    {
+        m_cliques.insert( clique, size);
+        break;
+    }
+    case 3:
+    {
+        if (! m_cliques.contains( clique) &&
+            ! m_cliques.contains( QString::number (this->name()) +
+                                  "," + members[2] +
+                                  "," + members[1] ) )
+        {
+            m_cliques.insert( clique, size);
+            return true ;
+        }
+        else
+            return false;
+        break;
+    }
+    case 4:
+    {
+        if (! m_cliques.contains( clique) &&
+                ! m_cliques.contains(  QString::number (this->name()) +
+                                       "," + members[1] +
+                                       "," + members[3] +
+                                       "," + members[2] ) &&
+                ! m_cliques.contains(  QString::number (this->name()) +
+                                       "," + members[2] +
+                                       "," + members[1] +
+                                       "," + members[3] ) &&
+                ! m_cliques.contains(  QString::number (this->name()) +
+                                       "," + members[2] +
+                                       "," + members[3] +
+                                       "," + members[1] ) &&
+                ! m_cliques.contains(  QString::number (this->name()) +
+                                       "," + members[3] +
+                                       "," + members[1] +
+                                       "," + members[2] ) &&
+                ! m_cliques.contains(  QString::number (this->name()) +
+                                       "," + members[3] +
+                                       "," + members[2] +
+                                       "," + members[1] )
+                )
+        {
+            m_cliques.insert( clique, size);
+            return true ;
+        }
+        else
+        {
+            return false;
+        }
+
+        break;
+    }
+    };
+    return false;
 }
 
 
-void Vertex::setName (long int v1) {
-	m_name=v1; 
-}
-
-
-
-
-void Vertex::setEnabled (bool flag ){
-	m_enabled=flag;
-}
-
-bool Vertex::isEnabled (){
-	return m_enabled;
-}
-
-
-
-void Vertex::setSize(int size) {
-    qDebug() << " Vertex::setSize() : " << size;
-	m_size=size;
-}
-
-int  Vertex::size(){
-	return m_size;
-}
-
-
-void Vertex::setShape(QString shape){
-	m_shape=shape;
-}
-
-QString Vertex::shape(){
-	return m_shape;	
-}
-
-
-void Vertex::setColor(QString color){
-	m_color=color;
-}
-
-QString Vertex::color(){
-	return m_color;	
-}
-
-
-
-
-void Vertex::setNumberColor (QString color) {
-	m_numberColor = color;
-}
-
-
-QString Vertex::numberColor(){
-	return m_numberColor;
-}
-
-
-void Vertex::setNumberSize (int size){
-	m_numberSize=size;
-}
-
-
-int Vertex::numberSize(){
-	return m_numberSize;
-}
-
-
-
-void Vertex::setLabel(QString label){
-	m_label=label;
-}
-
-QString Vertex::label(){
-	return m_label;	
-}
-
-void Vertex::setLabelColor(QString labelColor){
-	m_labelColor=labelColor;
-}
-
-QString Vertex::labelColor(){
-	return m_labelColor;	
-}
-
-
-void Vertex::setLabelSize(int newSize){
-	m_labelSize=newSize;
-}
-
-int Vertex::labelSize(){
-	return m_labelSize;
-}
-
-
-void Vertex::setX(float  x){
-//	qDebug("Vertex setX with %f",x);
-	m_x=x;
-}
-
-float Vertex::x(){
-	return m_x;	
-}
-
-
-void Vertex::setY(float y){
-//	qDebug("Vertex setY with %f", y);
-	m_y=y;
-}
-
-float	Vertex::y(){
-	return m_y;	
-}
-
-
-QPointF Vertex::pos () { 
-	return QPointF ( x(), y() ); 
-}
-
-
-
-void Vertex::setOutLinkColor(long int target, QString color){
-	qDebug()<<"Vertex: update linkColor to vertex "<< target<< " color: "<< color;
-	outLinkColors[target]=color;
-}
-
-//FIXME: See MW line 1965
-// FIXME MULTIGRAPH
-QString Vertex::outLinkColor(int target){
-	if (outLinkColors.contains(target))
-		return outLinkColors.value(target);	
-	else return "black";
-}
 
 
 void Vertex::clearPs()	{  
@@ -743,13 +795,15 @@ ilist Vertex::Ps(void) {
 	 return myPs;
 }
 
+
+
 Vertex::~Vertex() {
     qDebug() << " Vertex:: destroying my data";
-    m_outLinks.clear();
+    m_outEdges.clear();
     outLinkColors.clear();
     clearPs();
-    m_outLinks.clear();
-    m_inLinks.clear();
+    m_outEdges.clear();
+    m_inEdges.clear();
 }
 
 
