@@ -1,11 +1,11 @@
 /***************************************************************************
- SocNetV: Social Networks Visualizer
- version: 1.4
+ SocNetV: Social Network Visualizer
+ version: 1.9
  Written in Qt
  
                          webcrawler.h  -  description
                              -------------------
-    copyright            : (C) 2005-2014 by Dimitris B. Kalamaras
+    copyright            : (C) 2005-2015 by Dimitris B. Kalamaras
     email                : dimitris.kalamaras@gmail.com
  ***************************************************************************/
 
@@ -29,41 +29,63 @@
 
 using namespace std;
 
-#include <QThread>
 #include <QNetworkReply>
+#include <QUrl>
 
-class Reader : public QThread  {
-	Q_OBJECT
-public:  
-	void createNode(QString url, bool enqueue_to_frontier);
-	void createEdge (int source, int target);
-
-public slots:
-    void load(QNetworkReply*);
-signals:
-	void signalCreateNode(QString url, int no);
-	void signalCreateEdge (int source, int target);
-	
-protected:
-	void run();
-};
+class QNetworkAccessManager;
+class QNetworkRequest;
 
 
-class WebCrawler :  public QThread {
-	Q_OBJECT
+
+class WebCrawler_Parser : public QObject  {
+    Q_OBJECT
 public:
-	void load(QString seed, int maxNodes, int maxRecursion, bool goOut);
-	void terminateReaderQuit ();
+    WebCrawler_Parser(QString seed, int maxNodes, int maxLinksPerPage,
+                      bool extLinks, bool intLinks);
+    ~WebCrawler_Parser();
 public slots:
-	void slotCreateNode(QString url, int no);
-	void slotCreateEdge (int source, int target);
+    void parse(QNetworkReply *reply);
+    void newLink(int s, QUrl target, bool enqueue_to_frontier);
 signals:
-	void createNode(QString url, int no);
-	void createEdge (int source, int target);
-protected:
-	void run();
-private: 
-	QString url;
-	Reader reader;
+    void signalCreateNode(QString url, int no);
+    void signalCreateEdge (int source, int target);
+    void startSpider();
+    void finished (QString);
+private:
+    QByteArray ba;
+    QMap <QUrl, int> knownUrls;
+    QUrl  m_seed;
+    int m_maxPages;
+    int m_discoveredNodes;
+    int m_maxLinksPerPage;
+    bool m_extLinks, m_intLinks;
 };
+
+
+class  WebCrawler_Spider : public QObject  {
+    Q_OBJECT
+public:
+    WebCrawler_Spider(QString seed, int maxNodes, int maxLinksPerPage
+                      ,bool extLinks, bool intLinks);
+    ~WebCrawler_Spider();
+public slots:
+    void get();
+    void httpFinished(QNetworkReply *reply);
+
+signals:
+    void parse(QNetworkReply *reply);
+    void finished (QString);
+private:
+    QNetworkAccessManager *http;
+    QNetworkRequest *request;
+    QNetworkReply *reply;
+    QUrl currentUrl ;
+    QString  m_seed;
+    int m_maxPages;
+    int m_visitedNodes;
+    int m_maxLinksPerPage;
+    bool m_extLinks, m_intLinks;
+
+};
+
 #endif
