@@ -1,12 +1,15 @@
 /***************************************************************************
- SocNetV: Social Networks Visualizer
- version: 0.90
- Written in Qt 4.4
+ SocNetV: Social Network Visualizer
+ version: 1.9
+ Written in Qt
  
                          webcrawlerdialog.h  -  description
                              -------------------
-    copyright            : (C) 2005-2010 by Dimitris B. Kalamaras
+    copyright            : (C) 2005-2015 by Dimitris B. Kalamaras
     email                : dimitris.kalamaras@gmail.com
+    website:             : http://dimitris.apeiro.gr
+    project site         : http://socnetv.sourceforge.net
+
  ***************************************************************************/
 
 /*******************************************************************************
@@ -39,9 +42,11 @@ class WebCrawlerDialog: public QDialog
 public:
 	WebCrawlerDialog (QWidget *parent = 0);
 public slots:
+    void checkErrors ();
 	void gatherData ();
 signals:
-	void userChoices( QString, int, int, bool);	
+    void userChoices( QString, int, int, bool, bool);
+    void webCrawlerDialogError(QString);
 private:
 	Ui::WebCrawlerDialog ui;
 
